using System;

namespace stream {

	/// <summary>
	/// Rajapinta filttereille
	/// </summary>
	/// <remarks> Miika Nurminen, 16.9.2004</remarks>
	public interface Filter {

		/// <summary>
		/// Ksittelee sytevirrasta tulevan rivin.
		/// </summary>
		/// <param name="s">sytevirran rivi</param>
		/// <returns>
		/// tulostevirtaan tulostettava rivi. Jos null, mitn ei tulosteta.
		/// </returns>

		string processLine(string s);
		/// <summary>
		/// Metodia kutsutaan, kun sytevirta on lopussa.
		/// </summary>
		/// <returns>
		/// tulostevirtaan tulostettava rivi. Jos null, mitn ei tulosteta.
		/// </returns>
		string getResult();
	}

	/// <summary>
	/// Kaiutin
	/// </summary>
	/// <remarks> Miika Nurminen, 16.9.2004</remarks>
	public class EchoFilter : Filter {

		public String processLine(string s) {
			return s;
		}

		public string getResult() {
			return null;
		}
	}

	/// <summary>
	/// Summaa tietovirrassa olevia numeroita
	/// </summary>
	/// <remarks> Miika Nurminen, 16.9.2004</remarks>
	public class Summaaja : Filter {
		private int currentSum = 0;

		public string processLine(string s) {
			try {
			int i = Convert.ToInt32(s);
			currentSum += i;
		}
		catch (Exception n) {
			Console.Error.WriteLine(n.Message);
		}
		return null;
  }

		public string getResult() {
			return "Summa: "+Convert.ToString(currentSum);
		}
  }

}
