using System;

// Huom! C#:n ja Javan sililuokat ovat lhes identtiset nimi myten!
using System.Collections;

namespace stream
{
	/// <summary>
	/// lukee dataa |-erotetusta sytteest, esimerkki siliiden kytst
	/// </summary>
	/// <remarks> Miika Nurminen, 16.9.2004</remarks>
	public class Lukija : Filter {
		// ArrayList, joka sislt string-taulukoita (string[])
		private ArrayList content = new ArrayList();
		private ContentComparer comparer;

		public Lukija(int fieldNum) {
			comparer = new ContentComparer(fieldNum);
		}

		public string processLine(string s) {
			/* C#:ss ei ole StringTokenizer-luokkaa, mutta split-metodi
			   ajaa saman asian */
			char[] separators = {'|'};
			string[] values = s.Split(separators);
			content.Add(values);
			return null;
		  }

		public string getResult() {
			sort();
			return ToString();
		}


		/// <summary>
		/// Vertailijaluokka. Huom. C#:ss sisluokat eivt pse
		/// automaattisesti ulkoisen luokan tietoihin, kuten Javassa.
		/// </summary>
		public class ContentComparer : IComparer {
			private int field;
			public ContentComparer(int num) {
				fieldNum = num;
			}
			public int Compare( Object x, Object y )  {
				string[] a = (string[])x;
				string[] b = (string[])y;
				return a[fieldNum].CompareTo(b[fieldNum]);
			}

			// Esimerkki propertysta
			public int fieldNum {
				get { return field; }
				set { field = value; }
			}
		}

		private void sort() {
			content.Sort(comparer);
		}

		/// <summary>
		/// jrjest taulukon n:nnen kentn mukaan. huom. ei turvallinen
		/// (ei rajatarkastuksia) - lisksi tietoja luettaessa kenttien
		/// mr voi vaihdella.
		/// </summary>
		public void sortByField(int i) {
			comparer.fieldNum = i;
			sort();
		}

		/// <summary>
		/// summaa n:nnen kentn. huom, oletuksena on, ett kentt
		/// sislt numerotietoa. ei rajatarkistuksia!
		/// </summary>
		public int sumByField(int i) {
			int result = 0;
			foreach (String[] s in content) {
				try {
					result += Convert.ToInt32(s[i]);
				}
				catch (Exception e) {
					Console.Error.WriteLine(e.Message);
				}
			}
			return result;
		}

		/// <summary>
		/// Syrjytetn ylluokan ToString
		/// </summary>
		/// <returns>
		/// Taulukon merkkijonoesitys
		/// </returns>
		public override string ToString() {
			// StringBuilder == Javan StringBuffer
			System.Text.StringBuilder sb = new System.Text.StringBuilder();
			foreach (String[] s in content) {
				for (int i=0; i<s.Length; i++) {
					if (i>0) sb.Append('|');
					sb.Append(s[i]);
				}
				sb.Append('\n');
			}
			return sb.ToString();
		}
	}
}
